from .lenet import LeNet
from .resnet import ResNet
from .alexnet import AlexNet
from .minivggnet import MiniVGGNet
from .shallownet import ShallowNet
from .minigooglenet import MiniGoogLeNet
from .mxagegendernet import MxAgeGenderNet
from .deepergooglenet import DeeperGoogLeNet

from keras.applications import VGG16
from keras.applications import VGG19
from keras.applications import Xception
from keras.applications import ResNet50
from keras.applications import InceptionV3